searchData={"content_type":"text/markdown","items":[{"doc":"Top-level module for Etso.","ref":"Etso.html","title":"Etso","type":"module"},{"doc":"","ref":"Etso.html#t:repo/0","title":"Etso.repo/0","type":"type"},{"doc":"","ref":"Etso.html#t:schema/0","title":"Etso.schema/0","type":"type"},{"doc":"","ref":"Etso.html#t:table/0","title":"Etso.table/0","type":"type"},{"doc":"Used as an Adapter in Repo modules, which transparently spins up one ETS table for each Schema\nused with the Repo, namespaced to the Repo to allow concurrent running of multiple Repositories.\n\nThe Etso Adapter implements the `Ecto.Adapter.Schema` and `Ecto.Adapter.Queryable` behaviours.\n\nTo use the Etso Adapter in your application, define a Repo like this:\n\n    defmodule MyApp.Repo do\n      @otp_app Mix.Project.config()[:app]\n      use Ecto.Repo, otp_app: @otp_app, adapter: Etso.Adapter\n    end","ref":"Etso.Adapter.html","title":"Etso.Adapter","type":"module"},{"doc":"Repo-level Supervisor which supports the ETS Adapter. Within the Supervision Tree, a Dynamic\nSupervisor is used to hold the Table Servers, and a Registry is used to keep track of both the\nTable Servers, and their ETS Tables.","ref":"Etso.Adapter.Supervisor.html","title":"Etso.Adapter.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Etso.Adapter.Supervisor.html#child_spec/1","title":"Etso.Adapter.Supervisor.child_spec/1","type":"function"},{"doc":"Starts the Supervisor for the given `repo`.","ref":"Etso.Adapter.Supervisor.html#start_link/1","title":"Etso.Adapter.Supervisor.start_link/1","type":"function"},{"doc":"Provides convenience function to spin up a Registry, which is used to hold the Table Servers\n(registered by GenServer when starting up), alongside their ETS tables (registered when the\nTable Server starts).","ref":"Etso.Adapter.TableRegistry.html","title":"Etso.Adapter.TableRegistry","type":"module"},{"doc":"Returns Child Specification for the Table Registry that will be associated with the `repo`.","ref":"Etso.Adapter.TableRegistry.html#child_spec/1","title":"Etso.Adapter.TableRegistry.child_spec/1","type":"function"},{"doc":"Returns the ETS table associated with the given `repo` which is used to hold data for `schema`.","ref":"Etso.Adapter.TableRegistry.html#get_table/2","title":"Etso.Adapter.TableRegistry.get_table/2","type":"function"},{"doc":"Registers the ETS table associated with the given `repo` which is used to hold data for `schema`.","ref":"Etso.Adapter.TableRegistry.html#register_table/3","title":"Etso.Adapter.TableRegistry.register_table/3","type":"function"},{"doc":"The Table Server is a simple GenServer tasked with starting and holding an ETS table, which is\nnamespaced in the Table Registry by the Repo and the Schema. Once the Table Server starts, it\nwill attempt to create the ETS table, and also register the ETS table with the Table Registry.","ref":"Etso.Adapter.TableServer.html","title":"Etso.Adapter.TableServer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Etso.Adapter.TableServer.html#child_spec/1","title":"Etso.Adapter.TableServer.child_spec/1","type":"function"},{"doc":"Starts the Table Server for the given `repo` and `schema`, with registration under `name`.","ref":"Etso.Adapter.TableServer.html#start_link/1","title":"Etso.Adapter.TableServer.start_link/1","type":"function"},{"doc":"Provides convenience function to spin up a Dynamic Supervisor, which is used to hold the Table\nServers.","ref":"Etso.Adapter.TableSupervisor.html","title":"Etso.Adapter.TableSupervisor","type":"module"},{"doc":"Returns Child Specification for the Table Supervisor that will be associated with the `repo`.","ref":"Etso.Adapter.TableSupervisor.html#child_spec/1","title":"Etso.Adapter.TableSupervisor.child_spec/1","type":"function"},{"doc":"Starts the Child under the Table Supervisor associated with the `repo`.","ref":"Etso.Adapter.TableSupervisor.html#start_child/2","title":"Etso.Adapter.TableSupervisor.start_child/2","type":"function"},{"doc":"The ETS Match Specifications module contains various functions which convert Ecto queries to\n[ETS Match Specifications](https://www.erlang.org/doc/apps/erts/match_spec.html) in order to\nexecute the given queries with ETS with as much pushed down to ETS as possible.\n\nThe basic shape of the match head is `[$1, $2, $3, …]` where each field is a named variable, the\nordering of the fields is determined by `Etso.ETS.TableStructure`.\n\nConditions are compiled according to the wheres in the underlying Ecto query, while the body is\ncompiled based on the selected fields in the underlying Ecto query.","ref":"Etso.ETS.MatchSpecification.html","title":"Etso.ETS.MatchSpecification","type":"module"},{"doc":"","ref":"Etso.ETS.MatchSpecification.html#build/2","title":"Etso.ETS.MatchSpecification.build/2","type":"function"},{"doc":"The ETS Objects Sorter module is responsible for sorting results returned from ETS according to\nthe sort predicates provided in the query.","ref":"Etso.ETS.ObjectsSorter.html","title":"Etso.ETS.ObjectsSorter","type":"module"},{"doc":"","ref":"Etso.ETS.ObjectsSorter.html#sort/2","title":"Etso.ETS.ObjectsSorter.sort/2","type":"function"},{"doc":"The ETS Table Structure module contains various convenience functions to aid the transformation\nbetween Ecto Schemas (maps) and ETS entries (tuples). The primary key is moved to the head, in\naccordance with ETS conventions. Composite primary keys can not be accepted, however.","ref":"Etso.ETS.TableStructure.html","title":"Etso.ETS.TableStructure","type":"module"},{"doc":"","ref":"Etso.ETS.TableStructure.html#entries_to_tuples/2","title":"Etso.ETS.TableStructure.entries_to_tuples/2","type":"function"},{"doc":"","ref":"Etso.ETS.TableStructure.html#field_names/1","title":"Etso.ETS.TableStructure.field_names/1","type":"function"},{"doc":"","ref":"Etso.ETS.TableStructure.html#fields_to_tuple/2","title":"Etso.ETS.TableStructure.fields_to_tuple/2","type":"function"},{"doc":"# Etso\n\n**Etso** is an [ETS][erlang-ets] adapter, allowing you to use `Ecto` schemas with ETS tables.\n\nWithin this library, a bare-bones Ecto Adapter is provided. The Adapter transparently spins up ETS tables for each Ecto Repo and Schema combination. The tables are publicly accessible to enable concurrency, and tracked by reference to ensure encapsulation. Each ETS table is spun up by a dedicated Table Server under a shared Dynamic Supervisor.\n\nFor a detailed look as to what is available, check out [Northwind Repo Test][northwind-repo-test].","ref":"readme.html","title":"Etso","type":"extras"},{"doc":"Key highlights of this library are:\n\n- It transparently handles translations between `Ecto.Schema` structs and ETS tuples.\n- It knows how to form ETS Match Specifications for simple queries.\n\nKey points to consider when adopting this library are:\n\n- It is suitable for rapid retrieval of simply formatted data, for example presets that do not change frequently.\n- It is not suitable for use as a full-fledged data layer unless your requirements are simple.","ref":"readme.html#highlights-benefits","title":"Highlights & Benefits - Etso","type":"extras"},{"doc":"The following features are working:\n\n- Basic query scenarios (C / R / U / D)\n  - Insert all (`c:Ecto.Repo.insert_all/3`, etc.)\n  - Insert one (`c:Ecto.Repo.insert/2`, etc.)\n  - Get all (`c:Ecto.Repo.all/2`, etc.)\n  - Get by ID (`c:Ecto.Repo.get/3`, etc.)\n  - Get where (`Ecto.Query.where/3`, etc.)\n  - Delete by ID (`c:Ecto.Repo.delete/2`, etc.)\n- Selects\n- Assocs\n- Preloads\n\nThe [Northwind Repo Test][northwind-repo-test] should give you a good idea of what’s included.\n\n#","ref":"readme.html#feature-coverage","title":"Feature Coverage - Etso","type":"extras"},{"doc":"The following features, for example, are missing:\n\n- Composite primary keys\n- Dynamic Repos (`c:Ecto.Repo.put_dynamic_repo/1`)\n- Aggregates (dynamic / static)\n- Joins\n- Windows\n- Transactions\n- Migrations\n- Locking","ref":"readme.html#missing-features","title":"Missing Features - Etso","type":"extras"},{"doc":"Using Etso is a two-step process. First, include it in your application’s dependencies:\n\n```elixir\ndefp deps do\n  [\n    {:etso, \"~> 1.1.0\"}\n  ]\nend\n```\n\nAfterwards, create a new `Ecto.Repo`, which uses `Etso.Adapter`:\n\n```elixir\ndefmodule MyApp.Repo do\n  @otp_app Mix.Project.config()[:app]\n  use Ecto.Repo, otp_app: @otp_app, adapter: Etso.Adapter\nend\n```\n\nOnce this is done, you can use any Schema against the Repo normally, as you would with any other Repo. Check out the [Northwind modules][northwind] for an example.","ref":"readme.html#installation","title":"Installation - Etso","type":"extras"},{"doc":"Originally, Etso was created to answer the question of whether ETS and Ecto can be married together. It has since found some uses in production applications, serving as a Data Repository for pre-defined nested content. This proved invaluable for rapid iteration.\n\n*If you have other Use Cases for this library, please add it here with a Pull Request.*\n\n- The [Erlef Website](https://github.com/erlef/website) is using Etso to cache and query news and event posts that are parsed and imported into the Repo on application startup.","ref":"readme.html#utilisation","title":"Utilisation - Etso","type":"extras"},{"doc":"This repository is extracted from a prior project [ETS Playground][evadne-ets-playground], which was created to support a session at ElixirConf EU 2019, [*Leveraging ETS Effectively.*][evadne-ets-deck]\n\nSpecifically, this library was created to illustrate the point that ETS can serve as a scalable storage layer for data which changes infrequently. Check out the [Northwind Importer][northwind-importer] for an example.","ref":"readme.html#further-note","title":"Further Note - Etso","type":"extras"},{"doc":"This project contains a copy of data obtained from the Northwind database, which is owned by Microsoft. It is included for demonstration and testing purposes only, and is excluded from the distribution. The Author thanks Microsoft Corporation for the inspiration.\n\nThe Author also wishes to thank the following individuals:\n\n- [Wojtek Mach][wojtekmach], for the [inspiration](https://github.com/wojtekmach/ets_ecto) regarding creation of an Ecto adapter for ETS.\n\n- [Steven Holdsworth](https://github.com/holsee), for initial concept proofing and refinement.\n\n- [Igor Kopestenski](https://github.com/laymer), for initial reviews.\n\n- [David Schainker](https://github.com/schainks), for initial reviews, and for finding uses for this library.\n\n- [Masanori Iwasaki][curi1119], for [support of `for…in` queries][pr-6].\n\n- [Kohei Noda][pobo380], for [fixing nested `{:orelse, …}` queries][pr-10].\n\n- [William Martins][wmartins], for [fixing primary key unicity check issues][gh-7].\n\n- [Soichiro Nishizawa][soichiro-nishizawa], for [providing the implementation of sorted results][gh-12].\n\n- [Doug W.][harmon25], for [providing insights into parallel preloads][gh-13].\n\n- [Atanda Rasheed][heywhy], for [support of JSON extract paths][gh-20].\n\n[erlang-ets]: http://erlang.org/doc/man/ets.html\n[northwind]: https://github.com/evadne/etso/tree/master/test/support/northwind\n[northwind-importer]: https://github.com/evadne/etso/tree/master/test/support/northwind/importer.ex\n[northwind-repo-test]: https://github.com/evadne/etso/blob/master/test/northwind/repo_test.exs\n[evadne-ets-playground]: https://github.com/evadne/ets-playground\n[evadne-ets-deck]: https://speakerdeck.com/evadne/leveraging-ets-effectively\n[wojtekmach]: https://github.com/wojtekmach\n[curi1119]: https://github.com/curi1119\n[pr-6]: https://github.com/evadne/etso/pull/6\n[pobo380]: https://github.com/pobo380\n[pr-10]: https://github.com/evadne/etso/pull/10\n[wmartins]: https://github.com/wmartins\n[gh-7]: https://github.com/evadne/etso/issues/7\n[soichiro-nishizawa]: https://github.com/soichiro-nishizawa\n[gh-12]: https://github.com/evadne/etso/issues/12\n[harmon25]: https://github.com/harmon25\n[gh-13]: https://github.com/evadne/etso/issues/13\n[heywhy]: https://github.com/heywhy\n[gh-20]: https://github.com/evadne/etso/issues/20","ref":"readme.html#acknowledgements","title":"Acknowledgements - Etso","type":"extras"}]}